#include "stdafx.h"
#include "matlabsarima.h"
#include "AdapterSArima.h"

bool  SArimaStarted = false;
bool  MLBEngineStarted = false;

//-----------------------------------------------------------------------------------
_DLLAPI int prepareSARIMA(void)
{
	if (!MLBEngineStarted)
	{
		MLBEngineStarted = mclInitializeApplication(nullptr, 0);
		if (!MLBEngineStarted)
		{
			std::cerr << "Could not initialize the Matlab Runtime (MCR)" << std::endl;
			return 0;
		}
	}	
	if (!SArimaStarted)
	{
		try
		{
			SArimaStarted = matlabsarimaInitialize();
			if (!SArimaStarted)
			{
				std::cerr << "Could not initialize the library properly" << std::endl;
				return false;
			}			
			return (SArimaStarted)?1:0;
		}
		catch (const mwException &e)
		{
			std::cerr << e.what() << std::endl;
			return -2;
		}
		catch (...)
		{
			std::cerr << "Unexpected error thrown" << std::endl;
			return -3;
		}
	}
	return 1;
}
//-----------------------------------------------------------------------------------

_DLLAPI void closeSARIMA(void)
{
	// FreeMemory();
	// Call the application and library termination routine	
	//if (SArimaStarted)
	{
		matlabsarimaTerminate();
		SArimaStarted = false;
	}
}
//-----------------------------------------------------------------------------------

_DLLAPI int goSarima(double *Res, double *DataSeries, int idx0, int nLoad, int iSeasonPeriod, int npredict, int filterOn, int PlotOn)
{	//
	// Memory for Res[] must be allocated. Length = nLoad+npredict !!!
	//
	if (!SArimaStarted)
	{
		SArimaStarted = (prepareSARIMA() > 0) ? true : false;
	}

	
	mwArray nSeries(1, 1, mxDOUBLE_CLASS), TimeHor(1, 1, mxDOUBLE_CLASS), MAlen(1, 1, mxDOUBLE_CLASS);
	mwArray SeasonLag(1, 1, mxDOUBLE_CLASS), DoPlot(1, 1, mxDOUBLE_CLASS), DoFilter(1, 1, mxDOUBLE_CLASS);

	if (SArimaStarted)
	{
		try
		{		
		    MAlen  = 20;         // MA averaging window						
			DoFilter  = (filterOn != 0) ? 1 : 0;
			TimeHor   = npredict; // prediction points
			SeasonLag = iSeasonPeriod;     // seasonality period for the SARIMA model			
			
			DoPlot = PlotOn;    // plot in testing mode

			nSeries = nLoad;    // data fragment of length nLoad
					
			mwArray Series(nLoad,1, mxDOUBLE_CLASS, mxREAL);
			mwArray Result; // result (filtered (if specified filterOn!=0) data and forecast)
			
		//	std::cout << " 2. MATLAB memory allocated. Press button\n"; 	getchar();
		//	prepare the data for loading into the MATLAB matrix
		//	for (int i = 0; i < nLoad; i++) data[i] = DataSeries[idx0 + i]; // fragment DataSeries[idx0: idx+nLoad]
			Series.SetData(&DataSeries[idx0], nLoad);
	
		//	std::cout << " 3. Matrix with data series ready. Press button\n";	getchar();

			ForecastBySARIMA(1, Result, nSeries, TimeHor, Series, SeasonLag, DoFilter, MAlen, DoPlot);
			size_t nres = Result.NumberOfElements();
			  			  Result.GetData(Res, nres);

		//	std::cout << "Forecast done. Full series  " << nres << "\n";	
						  return 0;
		}
		catch (const mwException& e)
		{
			std::cerr << e.what() << std::endl;
			return -2;
		}
		catch (...)
		{
			std::cerr << "Unexpected error thrown" << std::endl;
			return -3;
		}		
	}	
	return 0;
}